package de.hft.stuttgart.citydoctor2.gui;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import de.hft.stuttgart.citydoctor2.check.ExcludeFilterConfiguration;
import de.hft.stuttgart.citydoctor2.check.FilterConfiguration;
import de.hft.stuttgart.citydoctor2.check.IncludeFilterConfiguration;
import de.hft.stuttgart.citydoctor2.datastructure.FeatureType;
import de.hft.stuttgart.citydoctor2.gui.filter.TypeFilterSelection;
import de.hft.stuttgart.citydoctor2.utils.Localization;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TextField;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;

public class FilterPane {

	@FXML
	private VBox inclFeatureBox;

	@FXML
	private ImageView addInclFeatureView;

	@FXML
	private VBox inclMatchBox;

	@FXML
	private ImageView addInclMatchView;

	@FXML
	private Button addInclFeatureBtn;

	@FXML
	private Button addInclMatchBtn;

	@FXML
	private VBox exclFeatureBox;

	@FXML
	private ImageView addExclFeatureView;

	@FXML
	private VBox exclMatchBox;

	@FXML
	private ImageView addExclMatchView;

	@FXML
	private Button addExclFeatureBtn;

	@FXML
	private Button addExclMatchBtn;

	private ScrollPane pane;
	private Image deleteImg;

	private List<ComboBox<TypeFilterSelection>> includeTypeBoxes = new ArrayList<>();
	private List<TextField> includeMatcherFields = new ArrayList<>();

	private List<TextField> excludeMatcherFields = new ArrayList<>();

	private List<ComboBox<TypeFilterSelection>> excludeTypeBoxes = new ArrayList<>();

	private List<TypeFilterSelection> availableFilters;

	public FilterPane() throws IOException {
		FXMLLoader fxmlLoader = new FXMLLoader(FilterPane.class.getResource("FilterPane.fxml"));
		fxmlLoader.setController(this);
		pane = fxmlLoader.load();
	}

	public void initialize() {
		loadImages();

		availableFilters = new ArrayList<>();
		availableFilters.add(new TypeFilterSelection(FeatureType.BUILDING, Localization.getText("FilterPane.buildings")));
		availableFilters.add(new TypeFilterSelection(FeatureType.BRIDGE, Localization.getText("FilterPane.bridges")));
		availableFilters.add(new TypeFilterSelection(FeatureType.LAND, Localization.getText("FilterPane.landUse")));
		availableFilters.add(new TypeFilterSelection(FeatureType.TRANSPORTATION, Localization.getText("FilterPane.transportation")));
		availableFilters.add(new TypeFilterSelection(FeatureType.VEGETATION, Localization.getText("FilterPane.vegetation")));
		availableFilters.add(new TypeFilterSelection(FeatureType.WATER, Localization.getText("FilterPane.water")));
		setupIncludeFeatureButton();
		setupIncludeMatchButton();
		setupExcludeFeatureButton();
		setupExcludeMatchButton();
	}

	private void setupExcludeMatchButton() {
		addExclMatchBtn.setOnAction(ae -> addExcludeMatchField());
	}

	private TextField addExcludeMatchField() {
		TextField tf = new TextField();
		tf.setPrefWidth(400);
		excludeMatcherFields.add(tf);

		HBox box = new HBox(10);
		box.getChildren().add(tf);

		ImageView imgView = new ImageView(deleteImg);
		imgView.setFitHeight(25);
		imgView.setFitWidth(25);
		box.getChildren().add(imgView);
		imgView.setOnMouseClicked(me2 -> {
			excludeMatcherFields.remove(tf);
			exclMatchBox.getChildren().remove(box);
		});

		exclMatchBox.getChildren().add(box);
		return tf;
	}

	private void setupExcludeFeatureButton() {
		addExclFeatureBtn.setOnAction(ae -> addExcludeTypeFilterBox());
	}

	private ComboBox<TypeFilterSelection> addExcludeTypeFilterBox() {
		ComboBox<TypeFilterSelection> cBox = new ComboBox<>();
		cBox.setPrefWidth(400);
		cBox.getItems().addAll(availableFilters);
		cBox.getSelectionModel().select(0);
		excludeTypeBoxes.add(cBox);

		HBox box = new HBox(10);
		box.getChildren().add(cBox);

		ImageView imgView = new ImageView(deleteImg);
		imgView.setFitHeight(25);
		imgView.setFitWidth(25);
		box.getChildren().add(imgView);
		imgView.setOnMouseClicked(me2 -> {
			excludeTypeBoxes.remove(cBox);
			exclFeatureBox.getChildren().remove(box);
		});

		exclFeatureBox.getChildren().add(box);
		return cBox;
	}

	private void setupIncludeMatchButton() {
		addInclMatchBtn.setOnAction(ae -> addIncludeMatchField());
	}

	private TextField addIncludeMatchField() {
		TextField tf = new TextField();
		tf.setPrefWidth(400);
		includeMatcherFields.add(tf);

		HBox box = new HBox(10);
		box.getChildren().add(tf);

		ImageView imgView = new ImageView(deleteImg);
		imgView.setFitHeight(25);
		imgView.setFitWidth(25);
		box.getChildren().add(imgView);
		imgView.setOnMouseClicked(me2 -> {
			includeMatcherFields.remove(tf);
			inclMatchBox.getChildren().remove(box);
		});

		inclMatchBox.getChildren().add(box);
		return tf;
	}

	public void applyFilterConfig(FilterConfiguration filter) {
		includeMatcherFields.clear();
		inclMatchBox.getChildren().clear();
		includeTypeBoxes.clear();
		inclFeatureBox.getChildren().clear();
		
		excludeMatcherFields.clear();
		exclMatchBox.getChildren().clear();
		excludeTypeBoxes.clear();
		exclFeatureBox.getChildren().clear();
		applyExcludeFilters(filter);
		applyIncludeFilters(filter);
	}

	private void applyIncludeFilters(FilterConfiguration filter) {
		IncludeFilterConfiguration include = filter.getInclude();
		if (include != null) {
			for (String s : include.getIds()) {
				addIncludeMatchField().setText(s);
			}
			for (FeatureType type : include.getTypes()) {
				ComboBox<TypeFilterSelection> box = addIncludeTypeFilterBox();
				for (TypeFilterSelection tfs : box.getItems()) {
					if (tfs.getType() == type) {
						box.getSelectionModel().select(tfs);
					}
				}
			}
		}
	}

	private void applyExcludeFilters(FilterConfiguration filter) {
		ExcludeFilterConfiguration exclude = filter.getExclude();
		if (exclude != null) {
			for (String s : exclude.getIds()) {
				addExcludeMatchField().setText(s);
			}
			for (FeatureType type : exclude.getTypes()) {
				ComboBox<TypeFilterSelection> box = addExcludeTypeFilterBox();
				for (TypeFilterSelection tfs : box.getItems()) {
					if (tfs.getType() == type) {
						box.getSelectionModel().select(tfs);
					}
				}
			}
		}
	}

	private void setupIncludeFeatureButton() {
		addInclFeatureBtn.setOnAction(ae -> addIncludeTypeFilterBox());
	}

	private ComboBox<TypeFilterSelection> addIncludeTypeFilterBox() {
		ComboBox<TypeFilterSelection> cBox = new ComboBox<>();
		cBox.setPrefWidth(400);
		cBox.getItems().addAll(availableFilters);
		cBox.getSelectionModel().select(0);
		includeTypeBoxes.add(cBox);

		HBox box = new HBox(10);
		box.getChildren().add(cBox);

		ImageView imgView = new ImageView(deleteImg);
		imgView.setFitHeight(25);
		imgView.setFitWidth(25);
		box.getChildren().add(imgView);
		imgView.setOnMouseClicked(me2 -> {
			includeTypeBoxes.remove(cBox);
			inclFeatureBox.getChildren().remove(box);
		});

		inclFeatureBox.getChildren().add(box);
		return cBox;
	}

	public FilterConfiguration getConfiguration() {
		IncludeFilterConfiguration includeFilters = new IncludeFilterConfiguration();
		List<FeatureType> featureTypes = new ArrayList<>();
		for (ComboBox<TypeFilterSelection> box : includeTypeBoxes) {
			featureTypes.add(box.getSelectionModel().getSelectedItem().getType());
		}
		includeFilters.setTypes(featureTypes);

		List<String> ids = new ArrayList<>();
		for (TextField tf : includeMatcherFields) {
			ids.add(tf.getText());
		}
		includeFilters.setIds(ids);

		ExcludeFilterConfiguration excludeFilters = new ExcludeFilterConfiguration();

		List<FeatureType> excludeFeatureTypes = new ArrayList<>();
		for (ComboBox<TypeFilterSelection> box : excludeTypeBoxes) {
			excludeFeatureTypes.add(box.getSelectionModel().getSelectedItem().getType());
		}
		excludeFilters.setTypes(excludeFeatureTypes);

		List<String> excludeIds = new ArrayList<>();
		for (TextField tf : excludeMatcherFields) {
			excludeIds.add(tf.getText());
		}
		excludeFilters.setIds(excludeIds);
		
		
		FilterConfiguration fConfig = new FilterConfiguration();
		fConfig.setInclude(includeFilters);
		fConfig.setExclude(excludeFilters);
		return fConfig;
	}

	private void loadImages() {
		try {
			try (InputStream inStream = MainWindow.class.getResourceAsStream("icons/Add.png")) {
				Image img = new Image(inStream);
				addInclFeatureView.setImage(img);
				addInclMatchView.setImage(img);
				addExclFeatureView.setImage(img);
				addExclMatchView.setImage(img);
			}
			try (InputStream inStream = MainWindow.class.getResourceAsStream("icons/Delete.png")) {
				deleteImg = new Image(inStream);
			}
		} catch (IOException e) {
			// ignore close exception
		}
	}

	public ScrollPane getPane() {
		return pane;
	}

}
